<?php

class AMQP {
    // Ports
    const PORT = 5672;
    const SSL_PORT = 5671;
    
    public function __construct($host = null, $user = null) {
        $this->connection = new AMQPConnection();
        if ($host != null) {
            $this->setParameters($host, $user);
        } else {
            $this->host = $this->user = null;
        }
        $this->channel = null;
        $this->exchange = null;
        $this->queue = null;
        $this->port = AMQP::PORT;
    }
    
    public function setPort($port) {
        $this->port = $port;
    }
    
    public function setParameters($host, $user) {
        $this->host = $host;
        $this->user = $user;
        if ($this->host != null) {
            $this->connection->setHost($this->host);
        }
    }

    public function connect() {
        $this->connection->setLogin($this->user['name']);
        $this->connection->setPassword($this->user['password']);
        $this->connection->setPort($this->port);
        if ($this->port == AMQP::SSL_PORT) {
            $this->connection->setCACert("/usr/local/mgportal/certs/cacert.pem");
            $this->connection->setCert("/usr/local/mgportal/certs/client/cert.pem");
            $this->connection->setKey("/usr/local/mgportal/certs/client/key.pem");
            $this->connection->setVerify(false);
        }
        $this->connection->connect();
    }

    public function createChannel() {
        $this->channel = new AMQPChannel($this->connection);
        return $this->getChannel();
    }

    public function getChannel() {
        return $this->channel;
    }

    public function createExchange() {
        $this->exchange = new AMQPExchange($this->channel);
        return $this->getExchange();
    }

    public function getExchange() {
        return $this->exchange;
    }

    public function createQueue($routing_key) {
        $this->queue = new AMQPQueue($this->channel);
        $this->queue->setName($routing_key);
        $this->queue->setFlags(AMQP_NOPARAM);
        $this->queue->declareQueue();
        return $this->getQueue();
    }

    public function getQueue() {
        return $this->queue;
    }
    
    public function getConnection() {
        return $this->connection;
    }
    
    public function disconnect() {
        $this->connection->disconnect();
    }
}

?>
