<?php

class REST {
    const URL = "http://"; //localhost/api/";
    const SSL_URL = "https://";
    const ERROR_PREFIX = "Error: ";

    const OK = 200;
    const CREATED = 201;
    const ERROR = 500;

    function __construct($Config, $isLocal = true) {
        $this->auth_token = null;
        if ($isLocal) {
            $this->url = $Config->get_local_url(); 
            $this->ssl = false;
        } else {
            $this->url = "";
            $this->ssl = true;
        }
    }

    function run($command, $request = "GET", $host = Configuration::LOCALHOST, $data = null) {
        $curl = curl_init();
        if ($this->url === "") {
            if ($host === Configuration::LOCALHOST) {
                $url = REST::URL . $host . "/api/" . $command;                
            } else {
                $uri = $this->getURI($command);
                $url = REST::SSL_URL . $host . "/portal/api/" . $uri;
            }
        } else {
            $url = $this->url;
        }
        $url .= $command;
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        if ($this->ssl) {
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);           
        }
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $request);
        if ($request === "PUT" || $request === "POST") {
            curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                    'Content-Type: application/json',
                    'xg-token:' . $this->auth_token
            ));
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }
        //var_dump($url);
        //var_dump($request);
        //var_dump($this->auth_token);
        //var_dump($data);
        $result = curl_exec($curl);
        if ($result !== false) {
            // Convert result to array.
            $result = json_decode($result, true);
            $http_code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
            //var_dump($http_code);
            if ($http_code !== REST::OK && $http_code !== REST::CREATED) {
                if (is_array($result) && isset($result['message'])) {
                    $result = REST::ERROR_PREFIX . $result['message'];
                } else {
                    $result = REST::ERROR_PREFIX . " undefined server error";
                }
            }
        } else {
            $result = REST::ERROR_PREFIX . curl_error($curl);;
        }
        //curl_close($curl);
        //printf("curl result\n");
        //var_dump($result);
        return $result;
    }
    
    public function isError($result) {
        $error = false;
        if (is_string($result) && strpos($result, REST::ERROR_PREFIX) === 0) {
            $error = true;
        }
        return $error;
    }
 
    
    private function getURI($command) {
        $uri = "";
        switch ($command) {
            case Configuration::CMD_PROTECT_INFO:
            case Configuration::CMD_PROTECTED_MONTHLY:
                $uri = "capacity.php?cmd=";
                break;
            case Configuration::CMD_DEVICE_CHECK:
                $uri = "devices.php?cmd=";
                break;
        }
        return $uri;
    }

    public function setAuthFromAsset($asset, $fromDevice = false) {
        if ($fromDevice) {
            $device_header = 'xg-device';
            $random = rand(0, 1000000);
            $token = sprintf("%d:%s:%s:%d", $random, $device_header, $asset, time());
            $token = base64_encode($token);
            $this->auth_token = $token;
        }
    }
}

?>
