<?php
/**
 *  Commands
 */
require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'database.php');

class Commands {
    public function __construct($command, $conn) {
        $this->command = $command;
        $this->conn = $conn;
    }

    public function getConnection() {
        return $this->conn;
    }

    public function run($timestamp, $asset_tag, $data) {
        global $capacity;
        switch ($this->command) {
            case Configuration::CMD_PROTECT_INFO:
                $this->save_protected_capacity($timestamp, $asset_tag, $data);
                break;
            case Configuration::CMD_PROTECTED_MONTHLY:
                $this->save_monthly($timestamp, $asset_tag, $data);
                break;
            case Configuration::CMD_DEVICE_CHECK:
                $this->save_device($timestamp, $asset_tag, $data);
                break;
        }
    }
    
    private function save_protected_capacity($timestamp, $asset_tag, $data) {
        global $Log;
        if ($this->conn == null) {
            $this->conn = pg_connect("port=5432 dbname=mgportal user=postgres connect_timeout=30");
        }

        if ($this->conn !== false) {
            if ($device_id = $this->save_device($timestamp, $asset_tag, $data)) {
                $gb_used = 0.0;
                if (isset($data['data'])) {
                    $values = $data['data'];
                    if (is_array($values) && count($values) > 0) {
                        $gb_used = $values[0]['used'];
                    }
                }
                $query = "INSERT INTO mgportal.protected (date, vm_count, server_count, app_count, all_replicated, vm_replicated, gb_used, device_id) ";
                $query .= "VALUES (%d, %d, %d, %d, %d, %d, %f, %d)";
                $pg = sprintf($query, $timestamp, $data['vm_count'], $data['server_count'], $data['app_count'],
                                $data['all_replicating'], $data['vm_replicating'], $gb_used, $device_id);
                pg_query($this->conn, $pg);
            }
        } else {
            $Log->logError("Error creating database connection, information not logged");
        }
    }

    private function save_monthly($timestamp, $asset_tag, $data) {
        global $Log;

        if ($this->conn == NULL) {
            $this->conn = pg_connect("port=5432 dbname=mgportal user=postgres connect_timeout=30");
        }

        if ($this->conn !== false) {
            if ($device_id = $this->save_device($timestamp, $asset_tag, $data)) {
                $query = "INSERT INTO mgportal.monthly (date, protected_assets_by_query, protected_assets_by_api, device_id) ";
                $query .= "VALUES (%d, %d, %d, %d)";
                $pg = sprintf($query, $timestamp, $data['protected_assets'], $data['protected_assets_api'], $device_id);
                pg_query($this->conn, $pg);
            }
        } else {
            $Log->logError("Error creating database connection, information not logged");
        }
    }

    private function save_device($timestamp, $asset_tag, $data) {
        global $Log;

        $device_id = false;
        if ($this->conn == NULL) {
            $this->conn = pg_connect("port=5432 dbname=mgportal user=postgres connect_timeout=30");
        }

        if ($this->conn !== false) {
            $device_id = $this->is_device_present($asset_tag);
            if (!$device_id) {
                $licenseClass = $data["license_class"];
                $marketingName = $data["license_mkt_name"];
                $featureString = $data["license_feature"];
                $installDate = $data["install_date"];
                $hasSupport = isset($data["has_support_contract"]) ? ($data["has_support_contract"] === true ? "TRUE" : "FALSE") : "FALSE";
                $query = "INSERT INTO mgportal.devices (unique_identifier, has_support_contract, license_class, license_mkt, license_features, install_date, date_created)";
                $query .= " VALUES ('%s', %s, '%s', '%s', '%s', '%s', '%d')";
                $query .= " RETURNING id;";
                $pg = sprintf($query, $asset_tag, $hasSupport, $licenseClass, $marketingName, $featureString, $installDate, $timestamp);
                $result = pg_query($this->conn, $pg);
                if ($result !== false) {
                    $row = pg_fetch_row($result);
                    $device_id = $row[0];
                } else {
                    $Log->logError("Error saving device information");
                }
            }
        } else {
            $Log->logError("Error creating database connection, information not logged");
        }       
        return $device_id;
    }

    private function is_device_present($asset_tag) {
        $device_id = false;
        if ($this->conn !== false) {
            $result = pg_query($this->conn, "SELECT * from mgportal.devices");
            if ($result !== false) {
                $found = false;
                while ($row = pg_fetch_row($result)) {
                    if ($asset_tag == $row[1]) {
                        $found = true;
                        $device_id = $row[0];
                        break;
                    }
                }
            }
        }
        return $device_id;
    }
};

?>