#!/usr/bin/php
<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const VENDOR_DIR = "vendor/";
const CONFIG_FILE = "mgportal.conf";

require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'AMQP.php');
require_once(BASE_DIR . BIN_DIR . 'REST.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');

$Log = new Logger(Logger::DEBUG);
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "client");
$Config->load_configuration();
$REST = new REST($Config);
$Log->setLogLevel($Config->getLogLevel());

/*
 * Check to see if monitoring is turned off and exit if so.
 */
if (!$Config->anyDeviceMonitoring()) {
    $Log->logInfo("This device is not being managed or monitored");
    exit(-1);
} else if ($Config->anyDeviceManagement()) {
    // check rpms
    $output = shell_exec("rpm -q librabbitmq php-pecl-amqp");
    if (strstr($output, "librabbitmq") && strstr($output, "php-pecl-amqp")) {
        // all okay, packages present.
    } else {
        $Log->logError("Management cannot be performed until dependent packages, librabbitmq and php-amqp, are installed");
        exit(-1);
    }
}

$vendor = null;
$vendorName = $Config->getVendor();
if ($vendorName !== null) {
    require_once(BASE_DIR . BIN_DIR . VENDOR_DIR . $vendorName . '.php');
    $vendor = new Vendor($REST, $Config);
}
if ($vendor === null) {
    $Log->logError("Vendor information not found; please check config file for 'vendor'");
    exit(-1);
}

/*
 * Get the asset tag for routing key.
 */
$license = $REST->run('license');
if ($license === false || !isset($license['asset_tag'])) {
    $Log->logError("Error getting device asset");
    exit(-1);
}
$asset = $license['asset_tag'];

if ($asset !== false) {
    $manager = $Config->getFirstManager();
    
    if ($manager !== null) {
        try {
            $routing_key = $Config->generate_route('cloud', $asset);
            
            $AMQP = new AMQP($manager['host'], $Config->getUser($manager['host']));
            $AMQP->setPort($Config->getPort());
            $AMQP->connect();
            $channel = $AMQP->createChannel();
            $exchange = $AMQP->createExchange();

            $queue = new AMQPQueue($channel);
            $queue->setName($routing_key);
            $queue->setFlags(AMQP_NOPARAM);
            $queue->declareQueue();
            
            printf("Waiting on messages....\n");
            $queue->consume(function(AMQPEnvelope $envelope, AMQPQueue $queue) use ($exchange) {
                $returnValue = processMessage($envelope);
                if (is_array($returnValue)) {
                    $command = $returnValue['meta_command'];
                    $encoded_message = json_encode($returnValue);
                    printf("Publish response to command %s\n", $command, $envelope->getReplyTo());
                } else {
                    printf("Error processing command, publishing error.\n", $envelope->getReplyTo());
                    $message = array('meta_command', 'error');
                    $encoded_message = json_encode($message);
                }
                $queue->ack($envelope->getDeliveryTag());
                $exchange->publish($encoded_message, $envelope->getReplyTo(), AMQP_NOPARAM);
                printf("Done, waiting on messages....\n");
            });
            
            $AMQP->disconnect();
        }catch(Exception $ex){
            $Log->logError($ex);
        }
    } else {
        $Log->logError('No active management hosts are available.');
    }
} else {
    $Log->logError('Cannot get asset tag for message routing.');
}


$count = 0;
function processMessage($envelope) {
    global $count;
    global $Log;
    $Log->logInfo("Received Message " . $count++ . $envelope->getBody());
    return processCommand($envelope->getBody());
}

function processCommand($command) {
    global $vendor;
    $result = false;
    printf("Command is %s\n", $command);
    $result = $vendor->run_command($command);
    return $result;
}

?>
