#!/usr/bin/php
<?php

const BASE_DIR = "/usr/local/mgportal/";
const BIN_DIR = "bin/";
const CONFIG_DIR = "conf.d/";
const VENDOR_DIR = "vendor/";
const API_DIR = "/var/www/html/api/includes/";
const CONFIG_FILE = "mgportal.conf";

require_once(BASE_DIR . BIN_DIR . 'config.php');
require_once(BASE_DIR . BIN_DIR . 'AMQP.php');
require_once(BASE_DIR . BIN_DIR . 'REST.php');
require_once(BASE_DIR . BIN_DIR . 'logger.php');

$Log = new Logger(Logger::DEBUG);
// Load configuration for the client (device).
$Config = new Configuration(BASE_DIR . CONFIG_DIR, CONFIG_FILE, "client");
$Config->load_configuration();
$REST = new REST($Config);
$Log->setLogLevel($Config->getLogLevel());

/*
 * Check to see if monitoring is turned off and exit if so.
 */
if (!$Config->anyDeviceMonitoring()) {
    $Log->logInfo("This device is not being managed or monitored");
    exit(-1);
} else if ($Config->anyDeviceManagement()) {
    // check rpms
    $output = shell_exec("rpm -q librabbitmq php-pecl-amqp");
    if (strstr($output, "librabbitmq") && strstr($output, "php-pecl-amqp")) {
        // all okay, packages present.
    } else {
        $Log->logError("Management cannot be performed until dependent packages, librabbitmq and php-amqp, are installed");
        exit(-1);
    }
}

$vendor = null;
$vendorName = $Config->getVendor();
if ($vendorName !== null) {
    require_once(BASE_DIR . BIN_DIR . VENDOR_DIR . $vendorName . '.php');
    $vendor = new Vendor($REST, $Config);
}
if ($vendor === null) {
    $Log->logError("Vendor information not found; please check config file for 'vendor'");
    exit(-1);
}

if ($argc == 2) {
    $command = $Config->get_command($argv[1]);
    if ($command !== false) {
        try {
            $message = $vendor->run_command($command);
            if ($message !== false) {
                // All messages need license information
                $license = $REST->run('license');
                $vendor->update_message($message, $license);
                // All messages must have asset tag.
                if (isset($message['asset_tag']) && $message['asset_tag'] !== false) {
                    $assetTag = $message['asset_tag'];
                    $encoded_message = json_encode($message);
                    
                    // Good so far.  Send the message to all defined hosts.
                    $hosts = $Config->getHosts();
                    foreach ($hosts as $entry) {
                        $host = $entry['host'];
                        $isManaged = $Config->isManagedByHost($host);
                        $Log->logDebug("Message: " . ($isManaged ? "(managed): " : "(not managed): ") . $encoded_message);
                        if ($isManaged && $Config->getAMQPMonitoring()) {
                            $routing_key = Configuration::DEVICE_MONITORING_ROUTE;
                            $AMQP = new AMQP($entry['host'], $Config->getUser($entry['host']));
                            $AMQP->setPort($Config->getPort());
                            $AMQP->connect();
                            $channel = $AMQP->createChannel();
                            $exchange = $AMQP->createExchange();
                            $AMQP->createQueue($routing_key);
                            if ($exchange !== null) {
                                $exchange->publish($encoded_message, $routing_key);
                            }
                            $AMQP->disconnect();
                        } else {
                            $portalREST = new REST($Config, false);
                            $portalREST->setAuthFromAsset($assetTag, true);
                            $result = $portalREST->run($command, "POST", $host, $encoded_message);
                            if ($portalREST->isError($result)) {
                                $Log->logError($result . " (Host: " . $host . ")");
                                break;
                            }
                        }
                    }
                } else {
                    $Log->logError("Cannot send a message with no asset tag");
                }
            } else {
                $message = "Error running command " . $command;
                $Log->logError($message);
            }
        } catch(Exception $ex){
            $Log->logError($ex);
        }        
    } else {
        $message = "Error: " . $argv[1] . " is not a valid command";
        $Log->logError($message);
    }
} else {
    usage();
}

function usage() {
    printf("to_cloud <command>\n");
}
