<?php

class vendor {
    public function __construct($REST, $Config) {
        $this->REST = $REST;
        $this->Config = $Config;
    }
    
    public function run_command($command) {
        $message = "";
        switch ($command) {
            case Configuration::CMD_PROTECT_INFO:
                $message = $this->get_protection_information();
                break;
            case Configuration::CMD_PROTECTED_MONTHLY:
                $message = $this->get_month_end();
                break;
            case Configuration::CMD_DEVICE_CHECK:
                // Every API gets the license and asset info, so we just need to initialize as an empty array here.
                $message = array();
                break;
            case Configuration::CMD_GET_JOBS:
                $message = $this->get_job_list();
                break;
        }

        // Add the standard header to the message (command name, timestamp, etc.)
        $this->Config->add_message_header($message, $command);
        
        return $message;
    }
    
    private function get_protection_information() {
        $system_capacity = $this->REST->run('reports/system_capacity');
        if ($this->REST->isError($system_capacity)) {
            $message = false;
            return $message;
        }
        $message = $system_capacity;
        if (isset($message['data'])) {
            $data = $message['data'];
            // $data is array of elements, get first.
            if (is_array($data)) {
                $data = $data[0];
            }
            $message['server_count'] = $message['vm_count'] = $message['app_count'] = 0;
            // Loop through all items, get the count of protected items.
            foreach ($data as $key => $value) {
                if (is_array($value)) {
                    if ($key == 'clients') {
                        $message['server_count'] += $this->get_protected_count($value);
                    } else if ($key == 'vms') {
                        $message['vm_count'] += $this->get_protected_count($value);
                    } else {
                        $message['app_count'] += $this->get_protected_count($value);
                    }
                }
            }
        }

        $replicating_assets = $this->REST->run('replication/assets');
        $replicating_count = 0;
        $replicating_vms = 0;
        if (isset($replicating_assets['replicating'])) {
            $assets = $replicating_assets['replicating'];
            foreach ($assets as $asset) {
                $replicating_count++;
                if ($this->is_vm($asset)) {
                    $replicating_vms++;
                }
            }
        }
        $message["all_replicating"] = $replicating_count;
        $message["vm_replicating"] = $replicating_vms;    
        return $message;
    }
    
    /*
     * Given an array of items, return the count of ones with a last_full backup.
     */
    private function get_protected_count($items) {
        $count = 0;
        foreach ($items as $asset) {
            if (isset($asset['last_full']) && $asset['last_full'] > 0) {
                $count++;
            }
        }
        return $count;
    }

    private function get_month_end() {
        $message = array();

        // Original method - Query given to MSPs.
        $conn = pg_connect("port=5432 dbname=bpdb user=postgres connect_timeout=30");
        $query = "select count(distinct s.instance_id) as protected_assets from bp.successful_backups s inner join bp.nodes n ";
        $query .= "using (node_no) inner join bp.application_instances ai using (instance_id) inner join bp.application_lookup al ";
        $query .= "using (app_id) where al.app_id in (50, 51, 40, 1) and (now()::date - 30) < (to_timestamp(s.start_time)::date);";
        $result = pg_query($conn, $query);
        if ($result !== false) {
            $row = pg_fetch_assoc($result);
            $message = $row;
        }

        // New method - BPL API
        // Past 30 days
        //$now = time();
        //$lastMonth = $now - (30 * 86400);
        // Use strtotime to go back to beginning of day 30 days ago.
        $lastMonthDate = date("Y-m-d", strtotime("-30 days"));
        $lastMonth = strtotime($lastMonthDate);

        // Only file-level, VMware, and Hyper-V backups
        // app ID 1, 40, 51, 52
        $backupTypes = array ("master", "incremental", "differential", "selective", "baremetal",
                                "vmware full", "vmware differential", "vmware incremental",
                                "hyperv full", "hyperv incremental");
        global $Log;
        $Log->logInfo("Get backups since " . $lastMonthDate . " timestamp(" . $lastMonth . ")");
        bp_bypass_cookie(3);
        $backups = bp_get_backup_status(array('start_time' => $lastMonth,
                                              'type' => $backupTypes));
        $unique_instances = array();
        $unique_clients = array();
        if ($backups !== false) {
            foreach ($backups as $backup) {
                // Backups must be successful (completed, not failed)
                // Count number of uniques.  For clients, we get client_id, otherwise we get instance_id.
                if (isset($backup['status']) && $backup['status'] != 2) {
                    if (isset($backup['instance_id'])) {
                        if (!in_array($backup['instance_id'], $unique_instances)) {
                            $unique_instances[] = $backup['instance_id'];
                        }
                    } else if (isset($backup['client_id'])) {
                        if (!in_array($backup['client_id'], $unique_clients)) {
                            $unique_clients[] = $backup['client_id'];
                        }                       
                    }
                }
            }
        } else {
            $Log->logError("Error" . bp_error());
        }
        $uniques = count($unique_instances) + count($unique_clients);
        
        if (is_array($message)) {
            $message['protected_assets_api'] = $uniques;
        }
        return $message;
    }
    
    public function update_message(&$message, $license) {
        $message["asset_tag"] = $license['asset_tag'];
        $message["license_class"] = $license['class'];
        $message["license_mkt_name"] = $license["mkt_name"];
        $message["license_feature"] = $license['feature_string'];
        $message["install_date"] = $license["install_date"];
    }

    public function is_vm($asset) {
        $is_vm = false;

        $app_id = $asset['app_id'];

        switch ($app_id) {
            case 40:	// VMware
            case 50:	// Hyper-V
            case 51:
            case 52:
            case 140:	// Xen
                $is_vm = true;
                break;
        }

        return $is_vm;
    }
    
    private function get_job_list() {
        return $this->REST->run("jobs");
    }
}

?>
